from abc import abstractmethod
from typing import Iterable

import numpy as np
from sklearn.decomposition import PCA


class _EOF_base():

    def __init__(
        self,
        X: Iterable[np.ndarray],
        n_modes=None,
        norm=False
    ):

        if norm:
            X /= X.std(axis=0)

        self.n_samples = X.shape[0]
        self.n_features = X.shape[1]

        self.n_modes = n_modes
        if n_modes is None:
            self.n_modes = min(self.n_samples, self.n_features)

        # TODO: weights

        self.X = X

    def solve(self) -> None:
        '''
        Perform the EOF analysis.

        To boost performance, the standard solver is based on
        the PCA implementation of scikit-learn [1]_ which uses different algorithms
        to perform the decomposition based on the data matrix size.

        Naive approaches using singular value decomposition of the
        data matrix ``X (n x p)`` or the covariance matrix ``C (p x p)``
        quickly become infeasable computationally when the number of
        samples :math:`n` or features :math:`p` increase (computational power increases
        by :math:`O(n^2p)` and :math:`O(p^3)`, respectively.)


        References
        ----------
        .. [1] https://scikit-learn.org/stable/modules/generated/sklearn.decomposition.PCA.html

        '''

        pca = PCA(n_components=self.n_modes)
        self._pcs = pca.fit_transform(self.X)
        self._singular_values = pca.singular_values_
        self._explained_variance = pca.explained_variance_
        self._explained_variance_ratio  = pca.explained_variance_ratio_
        self._eofs = pca.components_.T

        # Consistent signs for deterministic output
        maxidx = [abs(self._eofs).argmax(axis=0)]
        flip_signs = np.sign(self._eofs[maxidx, range(self._eofs.shape[1])])
        self._eofs *= flip_signs
        self._pcs *= flip_signs

    def singular_values(self) -> np.ndarray:
        '''Get the singular values.

        The `i` th singular value :math:`\sigma_i` is defined by

        .. math::
           \sigma_i = \sqrt{n \lambda_i}

        where :math:`\lambda_i` and :math:`n` are the associated eigenvalues
        and the number of samples, respectively.

        '''

        return self._singular_values

    def explained_variance(self):
        '''Get the explained variance.

        The explained variance is simply given by the individual eigenvalues
        of the covariance matrix.

        '''

        return self._explained_variance

    def explained_variance_ratio(self):
        '''Get the explained variance ratio.

        The explained variance ratio is the fraction of total variance
        explained by a given mode and is calculated by :math:`\lambda_i / \sum_i^m \lambda_i`
        where `m` is the total number of modes.

        '''

        return self._explained_variance_ratio

    def eofs(self):
        '''Get the EOFs.

        The empirical orthogonal functions (EOFs) are equivalent to the eigenvectors
        of the covariance matrix of `X`.

        '''

        return self._eofs

    def pcs(self):
        '''Get the PCs.

        The principal components (PCs), also known as PC scores, are computed
        by projecting the data matrix `X` onto the eigenvectors.

        '''

        return self._pcs



from typing import Optional

import numpy as np
import pandas as pd

from ..models._eof_base import _EOF_base
from xeofs.pandas._dataframe_transformer import _DataFrameTransformer


class EOF(_EOF_base):
    '''EOF analysis of a single ``pd.DataFrame``.

    Parameters
    ----------
    X : pd.DataFrame
        Data to be decpomposed.
    n_modes : Optional[int]
        Number of modes to compute. Computing less modes can results in
        performance gains. If None, then the maximum number of modes is
        equivalent to ``min(n_samples, n_features)`` (the default is None).
    norm : bool
        Normalize each feature (e.g. grid cell) by its temporal standard
        deviation (the default is False).
    axis : int
        Axis along which variance should be maximsed (the default is 0).


    Examples
    --------

    Import package and create data:

    >>> import pandas as pd
    >>> from xeofs.pandas import EOF
    >>> rng = np.random.default_rng(7)
    >>> X = rng.standard_normal((4, 3))
    >>> df = pd.DataFrame(X)

    Initialize standardized EOF analysis and compute the first 2 modes:

    >>> model = EOF(df, norm=True, n_modes=2)
    >>> model.solve()

    Get explained variance:

    >>> model.explained_variance()
    ...     	explained_variance
    ...     mode
    ...     1	2.562701
    ...     2	1.167054

    Get EOFs:

    >>> model.eofs()
    ... mode	1	2
    ... 0	0.626041	-0.428431
    ... 1	0.677121	-0.115737
    ... 2	0.386755	0.896132

    Get PCs:

    >>> model.pcs()
    ... mode	1	2
    ... 0	0.495840	-0.221963
    ... 1	-2.254508	-0.470420
    ... 2	1.516900	-0.876695
    ... 3	0.241768	1.569078

    '''

    def __init__(
        self,
        X: pd.DataFrame,
        n_modes : Optional[int] = None,
        norm : bool = False,
        axis : int = 0
    ):

        if(np.logical_not(isinstance(X, pd.DataFrame))):
            raise ValueError('This interface is for `pandas.DataFrame` only.')

        self._tf = _DataFrameTransformer()
        X = self._tf.fit_transform(X)

        super().__init__(
            X=X,
            n_modes=n_modes,
            norm=norm
        )
        self._mode_idx = pd.Index(range(1, self.n_modes + 1), name='mode')

    def singular_values(self):
        svalues = super().singular_values()
        svalues = pd.DataFrame(
            svalues,
            columns=['singular_values'],
            index=self._mode_idx
        )
        return svalues

    def explained_variance(self):
        expvar = super().explained_variance()
        expvar = pd.DataFrame(
            expvar,
            columns=['explained_variance'],
            index=self._mode_idx
        )
        return expvar

    def explained_variance_ratio(self):
        expvar = super().explained_variance_ratio()
        expvar = pd.DataFrame(
            expvar,
            columns=['explained_variance_ratio'],
            index=self._mode_idx
        )
        return expvar

    def eofs(self):
        eofs = super().eofs()
        eofs = self._tf.back_transform_eofs(eofs)
        eofs.columns = self._mode_idx
        return eofs

    def pcs(self):
        pcs = super().pcs()
        pcs = self._tf.back_transform_pcs(pcs)
        pcs.columns = self._mode_idx
        return pcs



from typing import Iterable, Optional, Union

import numpy as np
import xarray as xr

from ..models._eof_base import _EOF_base
from xeofs.xarray._dataarray_transformer import _DataArrayTransformer


class EOF(_EOF_base):
    '''EOF analysis of a single ``xr.DataArray``.

    Parameters
    ----------
    X : xr.DataArray
        Data to be decomposed.
    n_modes : Optional[int]
        Number of modes to compute. Computing less modes can results in
        performance gains. If None, then the maximum number of modes is
        equivalent to ``min(n_samples, n_features)`` (the default is None).
    norm : bool
        Normalize each feature (e.g. grid cell) by its temporal standard
        deviation (the default is False).
    dim : str
        Define the dimension which should considered for maximising variance.
        For most applications in climate science, temporal variance is
        maximised (also known as S-mode EOF analysis) i.e. the time dimension
        should be chosen. If spatial variance should be maximised
        (i.e. T-mode EOF analysis), set e.g. ``dim=['lon', 'lat']``
        (the default is ``time``).

    Examples
    --------

    Import package and create data:

    >>> import xarray as xr
    >>> from xeofs.xarray import EOF
    >>> da = xr.tutorial.load_dataset('rasm')['Tair']
    >>> da = xr.tutorial.load_dataset('air_temperature')['air']
    >>> da = da.isel(lon=slice(0, 3), lat=slice(0, 2))

    Initialize standardized EOF analysis and compute the first 2 modes:

    >>> model = EOF(da, norm=True, n_modes=2)
    >>> model.solve()

    Get explained variance:

    >>> model.explained_variance()
    ... xarray.DataArray'explained_variance'mode: 2
    ...     array([5.8630486 , 0.12335653], dtype=float32)
    ... Coordinates:
    ...     mode (mode) int64 1 2
    ... Attributes:
    ...     (0)

    Get EOFs:

    >>> model.eofs()
    ... xarray.DataArray 'EOFs' lon: 3 lat: 2 mode: 2
    ... array([[[ 0.4083837 , -0.39021498],
    ...         [ 0.40758175,  0.42474997]],
    ...
    ...        [[ 0.40875185, -0.40969774],
    ...         [ 0.40863484,  0.41475943]],
    ...
    ...        [[ 0.40776297, -0.4237887 ],
    ...         [ 0.40837321,  0.38450614]]])
    ... Coordinates:
    ...     lat (lat) float32 75.0 72.5
    ...     lon (lon) float32 200.0 202.5 205.0
    ...     mode (mode) int64 1 2
    ... Attributes:
    ...     (0)

    Get PCs:

    >>> model.pcs()
    ... xarray.DataArray 'PCs' time: 2920 mode: 2
    ... array([[-3.782707  , -0.07754549],
    ...        [-3.7966802 , -0.13775176],
    ...        [-3.7969239 , -0.05770111],
    ...        ...,
    ...        [-3.2584608 ,  0.3592216 ],
    ...        [-3.031799  ,  0.2055658 ],
    ...        [-3.0840495 ,  0.25031802]], dtype=float32)
    ... Coordinates:
    ...     time (time) datetime64[ns] 2013-01-01 ... 2014-12-31T18:00:00
    ...     mode (mode) int64 1 2
    ... Attributes:
    ...     (0)

    '''

    def __init__(
        self,
        X: xr.DataArray,
        n_modes : Optional[int] = None,
        norm : bool = False,
        dim: Union[str, Iterable[str]] = 'time'
    ):

        if(np.logical_not(isinstance(X, xr.DataArray))):
            raise ValueError('This interface is for `xarray.DataArray` only.')

        self._tf = _DataArrayTransformer()
        X = self._tf.fit_transform(X, dim=dim)

        super().__init__(
            X=X,
            n_modes=n_modes,
            norm=norm
        )
        self._mode_idx = xr.IndexVariable('mode', range(1, self.n_modes + 1))
        self._dim = dim

    def singular_values(self) -> xr.DataArray:
        svalues = super().singular_values()
        return xr.DataArray(
            svalues,
            dims=['mode'],
            coords={'mode' : self._mode_idx},
            name='singular_values'
        )

    def explained_variance(self) -> xr.DataArray:
        expvar = super().explained_variance()
        return xr.DataArray(
            expvar,
            dims=['mode'],
            coords={'mode' : self._mode_idx},
            name='explained_variance'
        )

    def explained_variance_ratio(self) -> xr.DataArray:
        expvar = super().explained_variance_ratio()
        return xr.DataArray(
            expvar,
            dims=['mode'],
            coords={'mode' : self._mode_idx},
            name='explained_variance_ratio'
        )

    def eofs(self) -> xr.DataArray:
        eofs = super().eofs()
        eofs = self._tf.back_transform_eofs(eofs)
        eofs.name = 'EOFs'
        return eofs

    def pcs(self) -> xr.DataArray:
        pcs = super().pcs()
        pcs = self._tf.back_transform_pcs(pcs)
        pcs.name = 'PCs'
        return pcs



from typing import Union, Iterable

import numpy as np
import warnings

warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")


class _ArrayTransformer():
    '''Transform any N-D ``np.ndarray`` into a 2-D ``np.ndarray``.

    The parameter ``axis`` allows to choose which axes of the original array
    should be transformed as the first axis (rows). All other axes will be reshaped
    into the second axis (columns)

    Handles NaNs by removing all columns and/or rows which contain NaNs only.
    Additional NaNs at individual positions are not allowed and will raise
    an exception.

    '''

    def __init__(self):
        pass

    def fit(self, X : np.ndarray, axis : Union[int, Iterable[int]] = 0):
        # Convert axis to list
        if isinstance(axis, int):
            axis = list([axis])

        all_axes = np.arange(len(X.shape))
        axis_samples = axis
        axis_features = [a for a in all_axes if a not in axis_samples]
        # Be sure to include all axes
        if not np.isin((axis_samples + axis_features), all_axes).all():
            err_msg = 'One or more of axes={:} do not exist.'.format(axis)
            raise ValueError(err_msg)
        new_axes_order = axis_samples + axis_features

        shape_X = X.shape
        shape_samples = np.array(X.shape)[axis_samples]
        shape_features = np.array(X.shape)[axis_features]

        n_samples = shape_samples.prod()
        n_features = shape_features.prod()

        # Reshape into 2D array
        X = X.transpose(new_axes_order)
        shape_X_t = X.shape
        X = X.reshape(n_samples, n_features)

        # Remove "all-NaN"-columns/rows
        nan_matrix = np.isnan(X)
        idx_valid_features = ~nan_matrix.all(axis=0)
        idx_valid_samples = ~nan_matrix.all(axis=1)

        n_valid_samples = idx_valid_samples.sum()
        n_valid_features = idx_valid_features.sum()

        self.all_axes = all_axes
        self.axis_samples = axis_samples
        self.axis_features = axis_features

        self.new_axes_order = new_axes_order

        self.shape_X = shape_X
        self.shape_X_t = shape_X_t
        self.shape_samples = shape_samples
        self.shape_features = shape_features

        self.n_samples = n_samples
        self.n_features = n_features

        self.idx_valid_samples = idx_valid_samples
        self.idx_valid_features = idx_valid_features

        self.n_valid_samples = n_valid_samples
        self.n_valid_features = n_valid_features
        return self

    def transform(self, X : np.ndarray):
        # Transpose data matrix X
        try:
            X = X.transpose(self.new_axes_order)
        except ValueError:
            err_msg = 'Expected {:} dimensions, but got {:} instead.'
            err_msg = err_msg.format(len(self.all_axes), len(X.shape))
            raise ValueError(err_msg)

        shape_features = list(X.shape[len(self.axis_samples):])
        if any(self.shape_features != shape_features):
            err_msg = 'Expected feature shape {:} but got {:} instead.'
            err_msg = err_msg.format(self.shape_features, shape_features)
            raise ValueError(err_msg)

        # Reshape data matrix X
        try:
            X = X.reshape(-1, self.n_features)
        except ValueError:
            err_msg = (
                'This should not happen. Please consider raising an issue '
                'on GitHub: https://github.com/nicrie/xeofs/issues'
            )
            raise ValueError(err_msg)

        # Remove NaNs of new data
        # Remove invalid samples
        X = X[~np.isnan(X).all(axis=1)]
        # Remove invalid features from fitted data
        X = X[:, self.idx_valid_features]
        # Security check: No NaNs should remain, otherwise invalid input data
        if np.isnan(X).any():
            raise ValueError('Invalid data: contains individual NaNs')
        return X

    def fit_transform(self, X : np.ndarray, axis : Union[int, Iterable[int]] = 0):
        return self.fit(X=X, axis=axis).transform(X)

    def back_transform(self, X : np.ndarray):
        if len(X.shape) > 2:
            raise ValueError('Data must be 2D to be back-transformed.')
        # Original shape
        Xrec = np.zeros((self.n_samples, self.n_features)) * np.nan
        # Insert data at valid locations
        valid_grid = np.ix_(self.idx_valid_samples, self.idx_valid_features)
        Xrec[valid_grid] = X
        # Inverse reshaping
        Xrec = Xrec.reshape(tuple(self.shape_X_t))
        # Inverse transposing
        return Xrec.transpose(np.argsort(self.new_axes_order))

    def back_transform_eofs(self, X : np.ndarray):
        if len(X.shape) > 2:
            raise ValueError('Data must be 2D to be back-transformed.')
        eofs = np.zeros((self.n_features, X.shape[1])) * np.nan
        eofs[self.idx_valid_features, :] = X
        return eofs.reshape(tuple(self.shape_features) + (X.shape[1],))

    def back_transform_pcs(self, X : np.ndarray):
        if len(X.shape) > 2:
            raise ValueError('Data must be 2D to be back-transformed.')
        pcs = np.zeros((self.n_samples, X.shape[1])) * np.nan
        pcs[self.idx_valid_samples, :] = X
        return pcs.reshape(tuple(self.shape_samples) + (X.shape[1],))



import numpy as np
import pandas as pd

from .. import models


class _DataFrameTransformer(models.eof._ArrayTransformer):
    '''`_ArrayTransformer` wrapper for `pandas.DataFrame`.

    '''

    def __init__(self):
        super().__init__()

    def fit(self, X : pd.DataFrame, axis : int = 0):
        self.index = X.index
        self.columns = X.columns
        super().fit(X=X.values, axis=axis)
        return self

    def transform(self, X : pd.DataFrame):
        return super().transform(X.values)

    def fit_transform(self, X : pd.DataFrame, axis : int = 0):
        return self.fit(X=X, axis=axis).transform(X)

    def back_transform(self, X : np.ndarray):
        df = super().back_transform(X)
        return pd.DataFrame(df, index=self.index, columns=self.columns)

    def back_transform_eofs(self, X : np.ndarray):
        eofs = super().back_transform_eofs(X)
        return pd.DataFrame(eofs, index=self.columns)

    def back_transform_pcs(self, X : np.ndarray):
        pcs = super().back_transform_pcs(X)
        return pd.DataFrame(pcs, index=self.index)



from typing import Union, Iterable
import numpy as np
import xarray as xr

from .. import models


class _DataArrayTransformer(models.eof._ArrayTransformer):
    '''`_ArrayTransformer` wrapper for `xarray.DataArray`.

    '''

    def __init__(self):
        super().__init__()

    def fit(self, X : xr.DataArray, dim : Union[str, Iterable[str]] = 'time'):
        # Convert dim to list
        if isinstance(dim, str):
            dim = list([dim])
        # Get axis position for each dim
        if np.isin(dim, X.dims).all():
            axis = [np.argmax(np.isin(X.dims, d)) for d in dim]
        else:
            err_msg = 'One or more of {:} is not a valid dimension.'
            err_msg = err_msg.format(dim)
            raise ValueError(err_msg)

        self.dims = X.dims
        self.dims_samples = dim
        self.dims_features = [d for d in self.dims if d not in dim]

        self.coords = X.coords

        super().fit(X=X.values, axis=axis)
        return self

    def transform(self, X : xr.DataArray):
        expected_dims = np.array(self.dims)
        provided_dims = np.array(X.dims)
        if np.isin(expected_dims, provided_dims).all():
            return super().transform(X.values)
        else:
            err_msg = 'Expected dimensions {:}, but got {:} instead.'
            err_msg = err_msg.format(expected_dims)
            raise ValueError(err_msg)

    def fit_transform(self, X : xr.DataArray, dim : Union[str, Iterable[str]] = 'time'):
        return self.fit(X=X, dim=dim).transform(X=X)

    def back_transform(self, X : np.ndarray):
        da = super().back_transform(X)
        return xr.DataArray(da, dims=self.dims)

    def back_transform_eofs(self, X : np.ndarray):
        da = super().back_transform_eofs(X)
        dims = self.dims_features + ['mode']
        coords = {k: c for k, c in self.coords.items() if k in self.dims_features}
        coords['mode'] = range(1, X.shape[1] + 1)
        return xr.DataArray(da, dims=dims, coords=coords)

    def back_transform_pcs(self, X : np.ndarray):
        da = super().back_transform_pcs(X)
        dims = self.dims_samples + ['mode']
        coords = {k: c for k, c in self.coords.items() if k in self.dims_samples}
        coords['mode'] = range(1, X.shape[1] + 1)
        return xr.DataArray(da, dims=dims, coords=coords)



from typing import Optional, Union, Iterable

import numpy as np

from xeofs.models._eof_base import _EOF_base
from xeofs.models._array_transformer import _ArrayTransformer

import warnings

warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")


class EOF(_EOF_base):
    '''EOF analysis of a single ``np.ndarray``.

    Parameters
    ----------
    X : np.ndarray
        Data to be decpomposed. ``X`` can be any N-dimensional array with the
        first dimension containing the variable whose variance is to be
        maximised. All remaining dimensions will be automatically reshaped to
        obtain a 2D matrix.
    n_modes : Optional[int]
        Number of modes to compute. Computing less modes can results in
        performance gains. If None, then the maximum number of modes is
        equivalent to ``min(n_samples, n_features)`` (the default is None).
    norm : bool
        Normalize each feature (e.g. grid cell) by its temporal standard
        deviation (the default is False).
    axis : Union[int, Iterable[int]]
        Axis along which variance should be maximised. Can also be
        multi-dimensional. For example, given a data array of dimensions
        ``(n x p1 x p2)`` with `n` time series at ``p1`` and ``p2`` different
        locations, ``axis=0`` will maximise `temporal` variance along ``n``.
        In contrast, ``axis=[1, 2]`` will maximise `spatial` variance along
        ``(p1 x p2)`` (the default is 0).


    Examples
    --------

    Import package and create data:

    >>> import numpy as np
    >>> from xeofs.models import EOF
    >>> rng = np.random.default_rng(7)
    >>> X = rng.standard_normal((4, 2, 3))

    Initialize standardized EOF analysis and compute the first 2 modes:

    >>> model = EOF(X, norm=True, n_modes=2)
    >>> model.solve()

    Get explained variance:

    >>> model.explained_variance()
    ... array([4.81848833, 2.20765019])

    Get EOFs:

    >>> model.eofs()
    ... array([[[ 0.52002146, -0.00698575],
    ...         [ 0.41059796, -0.31603563]],
    ...
    ...         [[ 0.32091783,  0.47647144],
    ...         [-0.51771611,  0.01380736]],
    ...
    ...         [[-0.28840959,  0.63341346],
    ...         [ 0.32678537,  0.52119516]]])

    Get PCs:

    >>> model.pcs()
    ... array([[ 1.76084189, -0.92030205],
    ...        [-2.13581896, -1.49704775],
    ...        [ 2.02091078,  0.65502667],
    ...        [-1.64593371,  1.76232312]])

    '''

    def __init__(
        self,
        X: np.ndarray,
        n_modes : Optional[int] = None,
        norm : bool = False,
        axis : Union[int, Iterable[int]] = 0
    ):

        self._tf = _ArrayTransformer()
        X = self._tf.fit_transform(X, axis=axis)

        super().__init__(
            X=X,
            n_modes=n_modes,
            norm=norm
        )

    def eofs(self):
        eofs = super().eofs()
        return self._tf.back_transform_eofs(eofs)

    def pcs(self):
        pcs = super().pcs()
        return self._tf.back_transform_pcs(pcs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xeofs', 'xeofs.models', 'xeofs.pandas', 'xeofs.xarray']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.22.2,<2.0.0',
 'pandas>=1.4.1,<2.0.0',
 'pooch>=1.6.0,<2.0.0',
 'scikit-learn>=1.0.2,<2.0.0',
 'xarray>=0.21.1,<0.22.0']

setup_kwargs = {
    'name': 'xeofs',
    'version': '0.2.0',
    'description': 'Collection of EOF analysis and related techniques for climate science',
    'long_description': '|badge1| |badge2| |badge3| |badge4| |badge5|\n\n.. |badge1| image:: https://img.shields.io/github/v/tag/nicrie/xeofs?label=Release\n    :alt: GitHub tag (latest SemVer)\n.. |badge2| image:: https://img.shields.io/github/workflow/status/nicrie/xeofs/CI\n   :alt: GitHub Workflow Status (event)\n.. |badge3| image:: https://readthedocs.org/projects/xeofs/badge/?version=latest\n   :target: https://xeofs.readthedocs.io/en/latest/?badge=latest\n   :alt: Documentation Status\n.. |badge4| image:: https://img.shields.io/pypi/dm/xeofs   \n    :alt: PyPI - Downloads\n.. |badge5| image:: https://codecov.io/gh/nicrie/xeofs/branch/main/graph/badge.svg?token=8040ZDH6U7\n    :target: https://codecov.io/gh/nicrie/xeofs\n\n=================================\nxeofs: EOF analysis and variants\n=================================\nEmpirical orthogonal function (EOF) analysis, commonly referred to as\nprincipal component analysis (PCA), is a popular decomposition\ntechnique in climate science. Over the years, a variety of variants\nhave emerged but the lack of availability of these different methods\nin the form of easy-to-use software seems to unnecessarily hinder the\nacceptance and uptake of these EOF variants by the broad climate science\ncommunity.\n\n************************\nGoal (work in progress)\n************************\nCreate a Python package that provides simple access to a variety of different\nEOF-related techniques through the popular interfaces of NumPy_, pandas_\nand xarray_.\n\n\n************************\nInstallation\n************************\nThe package can be installed via\n\n.. code-block:: ini\n\n  pip install xeofs\n\n************************\nDocumentation\n************************\nDocumentation_ is work in progress.\n\n.. _Documentation: https://xeofs.readthedocs.io/en/latest/\n\n************************\nCredits\n************************\n\n- General project structure: yngvem_\n- Testing data: xarray_ \\& pooch_\n\n\n\n.. _NumPy: https://www.numpy.org\n.. _pandas: https://pandas.pydata.org\n.. _xarray: https://xarray.pydata.org\n.. _yngvem: https://github.com/yngvem/python-project-structure\n.. _pooch: https://github.com/fatiando/pooch\n',
    'author': 'Niclas Rieger',
    'author_email': 'niclasrieger@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nicrie/xeofs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
